#!/usr/bin/ksh

function main
{
    parse_args $*
    bc_definitions
    i=1
    while [[ $i -le $expressions ]]
    do
	${expr[i]}
	i=$((i+1))
    done
}
 
s=1
t=0
u=0
v=1
 
# S + Tz
# ------
# U + Vz
 
S=10
T=11
U=12
V=13
 
function transform
{
    initialize $S $s
    initialize $T $t
    initialize $U $u
    initialize $V $v

    multiply 20  1 $T
    add      21 20 $S
 
    multiply 30  1 $V
    add      31 30 $U
 
    conjugate 32 31
    conjugate 33 32
    conjugate 34 33
 
    multiply 40 32 33
    multiply 41 40 34
 
    multiply 42 41 21
 
    multiply 43 41 31

    reduce 42 43
 
    display 42
    display 43
}

function invariant
{
    copy 50 1
    conjugate 51 50
    conjugate 52 51
    conjugate 53 52
    subtract 60 52 50
    subtract 61 53 51
    multiply 62 61 60
    multiply 63 62 62
    subtract 64 51 50
    subtract 65 52 51
    subtract 66 53 52
    subtract 67 50 53
    multiply 68 64 65
    multiply 69 68 66
    multiply 70 69 67
    conjugate 71 70
    conjugate 72 71
    conjugate 73 72
    multiply 74 72 71
    multiply 75 74 73
    multiply 80 75 70
    multiply 81 75 63
    reduce 81 80
    display 81
    display 80
}

function norm
{
    copy 50 1
    conjugate 51 50
    conjugate 52 51
    conjugate 53 52

    multiply 60 51 50
    multiply 61 52 60
    multiply 62 53 61

    display 62
}

function copy
{
    printf "z = c(%s,%s)\n" $1 $2
}

function initialize
{
    printf "z = i(%s,%s)\n" $1 $2
}

function multiply
{
    printf "z = m(%s,%s,%s)\n" $1 $2 $3
}

function add
{
    printf "z = a(%s,%s,%s)\n" $1 $2 $3
}

function subtract
{
    printf "z = s(%s,%s,%s)\n" $1 $2 $3
}

function conjugate
{
    printf "z = g(%s,%s)\n" $1 $2
}

function display
{
    printf "z = d(%s)\n" $1
}

function reduce
{
    printf "z = r(%s,%s)\n" $1 $2
}

function parse_args
{
    expressions=1
    variables=1

    for arg
    do
	if [[ $arg == -* ]]
	then
	    expr[expressions]=${arg#-}
	    expressions=$((expressions + 1))
	else
	    echo "$arg" |
	    
	    awk -vn=$variables '

                       { sub(/\[/,"");
                         sub(/\]/,"");
                         split($0,ar,/,/);
                         printf("a[%s]=%s\n", n, ar[1]);
                         printf("b[%s]=%s\n", n, ar[2]);
                         printf("e[%s]=%s\n", n, ar[3]);
                         printf("f[%s]=%s\n", n, ar[4]);
                         ++n;
                       }'

	    variables=$((variables + 1))
	fi
    done
}
 
function bc_definitions
{
    echo '

define i(o,x) {
    a[o] = x
    b[o] = 0
    e[o] = 0
    f[o] = 0
}

define c(o,i) {
    a[o] = a[i]
    b[o] = b[i]
    e[o] = e[i]
    f[o] = f[i]
}
 
define n(o,i) {
    a[o] = -a[i]
    b[o] = -b[i]
    e[o] = -e[i]
    f[o] = -f[i]
}
 
define a(o,i,j) {
    a[o] = a[i] + a[j]
    b[o] = b[i] + b[j]
    e[o] = e[i] + e[j]
    f[o] = f[i] + f[j]
}
 
define s(o,i,j) {
    a[o] = a[i] - a[j]
    b[o] = b[i] - b[j]
    e[o] = e[i] - e[j]
    f[o] = f[i] - f[j]
}
 
define m(o,i,j) {
    a[o] = a[i]*a[j] - 4*b[i]*f[j] - 2*e[i]*e[j] - 4*f[i]*b[j] - 16*f[i]*f[j]
    b[o] = a[i]*b[j] + b[i]*a[j] - 2*e[i]*f[j] - 2*f[i]*e[j]
    e[o] = a[i]*e[j] + 2*b[i]*b[j] + e[i]*a[j] + 8*b[i]*f[j] + 4*e[i]*e[j] + 8*f[i]*b[j] + 28*f[i]*f[j]
    f[o] = a[i]*f[j] + b[i]*e[j] + e[i]*b[j] + f[i]*a[j] + 4*e[i]*f[j] + 4*f[i]*e[j]
}
 
define g(o,i) {
    a[o] = a[i] + 4*e[i]
    b[o] = 3*b[i] + 10*f[i]
    e[o] = -e[i]
    f[o] = -b[i] - 3*f[i]
}
 
define d(i) {
    100000000000000 + i
    a[i]
    b[i]
    e[i]
    f[i]
}

define e(x,y) {
    auto t, a, b
    a = x
    b = y
    if (a == 0) return (b)
    if (b == 0) return (a)
    if (a < 0) a = -a
    if (b < 0) b = -b
    while (b != 0) {
        t = b
        b = a % b
        a = t
    }
    return (a)
} 

define r(i,j) {
    auto g
    g = e( a[i], b[i] )
    g = e(    g, e[i] )
    g = e(    g, f[i] )
    g = e(    g, a[j] )
    g = e(    g, b[j] )
    g = e(    g, e[j] )
    g = e(    g, f[j] )

    if (a[j] * g < 0) g = -g

    a[i] /= g
    b[i] /= g
    e[i] /= g
    f[i] /= g

    a[j] /= g
    b[j] /= g
    e[j] /= g
    f[j] /= g
}

'
}

main $* | bc | awk '

$0 ~ /^1000000000000/ {
    begin = NR;
    i = $0;
    sub(/1000000000000/,"",i);
}

NR == (begin+1) {
    printf("a[%s]=%s\n", i, $0);
}

NR == (begin+2) {
    printf("b[%s]=%s\n", i, $0);
}

NR == (begin+3) {
    printf("e[%s]=%s\n", i, $0);
}

NR == (begin+4) {
    printf("f[%s]=%s\n", i, $0);
}

 '
